/* =======================================================================
    DEFAULTS
========================================================================== */
.nav-tabs {
  display: block;
  margin: 0 auto;
  border: none; }

.nav-tabs > li > a,
.nav-tabs > li > a:hover,
.nav-tabs > li > a:focus {
  background: #333333;
  color: #ddd;
  border-radius: 4px 4px 0 0;
  border: 1px solid transparent;
  padding-top: 8px;
  padding-bottom: 8px;
  line-height: 20px; }

.tab-content {
  display: block;
  margin: 0 auto;
  clear: both;
  text-align: left;
  border: none;
  border-radius: 4px;
  background-color: #333; }

.tab-pane {
  padding: 15px 15px 25px;
  min-height: 200px; }

.tab-pane-desc {
  float: left;
  width: 250px; }

.tab-pane-desc h3 {
  margin-top: 5px; }

.tab-pane-desc p {
  width: 90%;
  margin: 10px 0;
  color: #666; }

.navbaricon {
  margin-top: -15px;
  margin-bottom: -15px; }

[class^="icon-"],
[class*=" icon-"] {
  background-image: url("../images/glyphicons-halflings.png"); }

.icon-white {
  background-image: url("../images/glyphicons-halflings-white.png"); }

.dropdown-menu li > a:hover > [class^="menu-icon-"],
.dropdown-menu li > a:hover > [class*=" menu-icon-"] {
  background-image: url("../images/menu/menu-icons-white.png"); }

[class^="icon16-"],
[class*=" icon16-"] {
  background-image: url("../images/glyphicons-config-white.png"); }

.ui-autocomplete-loading {
  background: white url("../images/loading16.gif") right center no-repeat; }

.browserDialog.busy .ui-dialog-buttonpane {
  background: url("../images/loading.gif") 10px 50% no-repeat !important; }

.ui-progressbar .ui-progressbar-overlay {
  background: url("../images/animated-overlay.gif"); }

.ui-dialog,
.ui-dialog-buttonpane {
  background: #eceadf url("../images/ui-bg_fine-grain_10_eceadf_60x60.png") 50% 50% repeat !important; }

.ui-accordion-content,
.ui-tabs-panel {
  background: #ededed none !important; }

.ui-widget {
  font-family: inherit;
  font-size: 1em; }

.ui-widget.ui-widget-content {
  border: none; }

.ui-state-default,
.ui-widget-content .ui-state-default,
.ui-widget-header .ui-state-default {
  background: #ffffff;
  border: 1px solid #CCCCCC; }

.ui-state-hover,
.ui-widget-content .ui-state-hover,
.ui-widget-header .ui-state-hover,
.ui-state-focus,
.ui-widget-content .ui-state-focus,
.ui-widget-header .ui-state-focus {
  background: #ffffff; }

.ui-state-active,
.ui-widget-content .ui-state-active,
.ui-widget-header .ui-state-active {
  background: #F7F7F7; }

.ui-state-highlight,
.ui-widget-content .ui-state-highlight,
.ui-widget-header .ui-state-highlight {
  background: #fbf9ee url("../images/ui-bg_glass_55_fbf9ee_1x400.png") 50% 50% repeat-x; }

.ui-state-error,
.ui-widget-content .ui-state-error,
.ui-widget-header .ui-state-error {
  background: #fef1ec url("../images/ui-bg_glass_95_fef1ec_1x400.png") 50% 50% repeat-x; }

.ui-icon,
.ui-widget-content .ui-icon {
  background-image: url("../images/ui-icons_222222_256x240.png"); }

.ui-widget-header .ui-icon {
  background-image: url("../images/ui-icons_222222_256x240.png"); }

.ui-state-default .ui-icon {
  background-image: url("../images/ui-icons_8c291d_256x240.png"); }

.ui-state-hover .ui-icon,
.ui-state-focus .ui-icon {
  background-image: url("../images/ui-icons_222222_256x240.png"); }

.ui-state-active .ui-icon {
  background-image: url("../images/ui-icons_8c291d_256x240.png"); }

.ui-state-highlight .ui-icon {
  background-image: url("../images/ui-icons_2e83ff_256x240.png"); }

.ui-state-error .ui-icon,
.ui-state-error-text .ui-icon {
  background-image: url("../images/ui-icons_cd0a0a_256x240.png"); }

.ui-widget-overlay {
  background: #aaaaaa url("../images/ui-bg_flat_0_aaaaaa_40x100.png") 50% 50% repeat-x; }

.ui-state-active a, .ui-state-active a:link, .ui-state-active a:visited {
  color: #140F06;
  text-decoration: none; }

.ui-state-default a, .ui-state-default a:link, .ui-state-default a:visited {
  color: #222;
  text-decoration: none; }

#content {
  width: 100%;
  margin: 10px;
  margin-left: auto;
  margin-right: auto;
  clear: both; }

#sub-menu-container {
  margin: 0; }

#sub-menu {
  clear: both;
  font-size: 12px;
  align-content: flex-end;
  text-align: center !important; }

#sub-menu a {
  clear-after: top;
  margin: 2px; }

.upgrade-notification {
  width: 90%;
  text-align: center;
  margin-left: auto;
  margin-right: auto;
  margin-bottom: 0; }

[class^="menu-icon-"], [class*=" menu-icon-"] {
  background: url("../images/menu/menu-icons-black.png");
  height: 16px;
  width: 16px;
  display: inline-block;
  position: relative;
  top: 2px;
  float: left; }

[class^="menu-icon-"], [class*=" menu-icon-"] {
  background: url("../images/menu/menu-icons-white.png");
  height: 16px;
  width: 16px;
  display: inline-block;
  position: relative;
  top: 2px;
  float: left; }

.menu-icon-addshow {
  background-position: 0 0; }

.menu-icon-anime {
  background-position: -21px 0; }

.menu-icon-backlog-view {
  background-position: -42px 0; }

.menu-icon-backlog {
  background-position: -63px 0; }

.menu-icon-bittorrent {
  background-position: -84px 0; }

.menu-icon-config-index {
  background-position: -105px 0; }

.menu-icon-config {
  background-position: -126px 0; }

.menu-icon-failed-download {
  background-position: -147px 0; }

.menu-icon-home {
  background-position: -168px 0; }

.menu-icon-manage {
  background-position: -189px 0; }

.menu-icon-manage-searches {
  background-position: -210px 0; }

.menu-icon-poster {
  background-position: -231px 0; }

.menu-icon-postprocess {
  background-position: -252px 0; }

.menu-icon-restart {
  background-position: -273px 0; }

.menu-icon-shutdown {
  background-position: -294px 0; }

.menu-icon-update {
  background-position: -315px 0; }

.menu-icon-viewlog-errors {
  background-position: -336px 0; }

.menu-icon-viewlog {
  background-position: -357px 0; }

.menu-icon-kodi {
  background-position: -378px 0; }

.menu-icon-help {
  background-position: -399px 0; }

[class^="submenu-icon-"], [class*=" submenu-icon-"] {
  background: url("../images/menu/menu-icons-black.png");
  height: 16px;
  width: 16px; }

.submenu-icon-anime {
  background-position: -21px 0; }

.submenu-icon-bittorrent {
  background-position: -84px 0; }

.submenu-icon-failed-download {
  background-position: -147px 0; }

.submenu-icon-restart {
  background-position: -273px 0; }

.submenu-icon-shutdown {
  background-position: -294px 0; }

.submenu-icon-kodi {
  background-position: -378px 0; }

.footer {
  color: #4e4e4e;
  text-align: center;
  font-size: 12px; }

.footer-highlight {
  color: #111;
  display: inline; }

/* =======================================================================
    root_dirs.mako
    ========================================================================== */
.rootdir-selectbox,
.rootdir-selectbox #rootDirs,
.rootdir-controls {
  width: 100%; }

.rootdir-selectbox {
  padding: 0 0 5px; }

.rootdir-controls {
  text-align: center; }

/* =======================================================================
    index.mako
    ========================================================================== */
.imgbanner .banner {
  border: 1px solid #ccc;
  overflow: hidden;
  height: 66px;
  overflow: hidden;
  border-radius: 8px;
  vertical-align: top;
  width: 360px;
  display: block;
  margin-left: auto;
  margin-right: auto; }

.imgsmallposter .small {
  height: 66px;
  overflow: hidden;
  border-radius: 3px;
  vertical-align: middle;
  width: 45px;
  border: 1px solid #ccc;
  margin-right: 5px; }

.search {
  margin-bottom: 10px; }

.ui-progressbar {
  height: 20px;
  line-height: 18px; }

.ui-progressbar .ui-progressbar-value {
  box-sizing: content-box !important; }

.progressbarText {
  position: absolute;
  top: 0;
  width: 100%;
  height: 100%;
  overflow: visible;
  text-align: center;
  text-shadow: 0 0 0.1em #fff;
  vertical-align: middle;
  font-size: 12px;
  color: #000000; }

.progress-100 {
  background-image: -moz-linear-gradient(#395f07, #2a4705) !important;
  background-image: linear-gradient(#395f07, #2a4705) !important;
  background-image: -webkit-linear-gradient(#395f07, #2a4705) !important;
  background-image: -o-linear-gradient(#395f07, #2a4705) !important;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

.progress-80 {
  background-image: -moz-linear-gradient(#a6cf41, #5b990d) !important;
  background-image: linear-gradient(#a6cf41, #5b990d) !important;
  background-image: -webkit-linear-gradient(#a6cf41, #5b990d) !important;
  background-image: -o-linear-gradient(#a6cf41, #5b990d) !important;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

.progress-60 {
  background-image: -moz-linear-gradient(#fad440, #f2a70d) !important;
  background-image: linear-gradient(#fad440, #f2a70d) !important;
  background-image: -webkit-linear-gradient(#fad440, #f2a70d) !important;
  background-image: -o-linear-gradient(#fad440, #f2a70d) !important;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

.progress-40 {
  background-image: -moz-linear-gradient(#fab543, #f2700d) !important;
  background-image: linear-gradient(#fab543, #f2700d) !important;
  background-image: -webkit-linear-gradient(#fab543, #f2700d) !important;
  background-image: -o-linear-gradient(#fab543, #f2700d) !important;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

.progress-20 {
  background-image: -moz-linear-gradient(#da5945, #b11a10) !important;
  background-image: linear-gradient(#da5945, #b11a10) !important;
  background-image: -webkit-linear-gradient(#da5945, #b11a10) !important;
  background-image: -o-linear-gradient(#da5945, #b11a10) !important;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

#container, #container-anime {
  margin: 0 auto; }

.show-container {
  margin: 10px;
  width: 188px;
  height: 352px;
  background-color: #DFDACF;
  border: 1px solid #111;
  border-radius: 6px; }

.show-image {
  overflow: hidden;
  width: 186px;
  height: 273px;
  border-top-left-radius: 5px;
  border-top-right-radius: 5px; }

.show-container .ui-progressbar {
  height: 7px !important;
  top: -2px; }

.show-container .ui-corner-all, .ui-corner-bottom, .ui-corner-right, .ui-corner-br {
  border-bottom-right-radius: 0; }

.show-container .ui-corner-all, .ui-corner-bottom, .ui-corner-left, .ui-corner-bl {
  border-bottom-left-radius: 0; }

.show-container .ui-corner-all, .ui-corner-top, .ui-corner-right, .ui-corner-tr {
  border-top-right-radius: 0; }

.show-container .ui-corner-all, .ui-corner-top, .ui-corner-left, .ui-corner-tl {
  border-top-left-radius: 0; }

.show-container .ui-widget-content {
  border-top: 1px solid #111;
  border-bottom: 1px solid #111;
  border-left: 0;
  border-right: 0; }

.ui-progressbar .ui-progressbar-value {
  height: 20px; }

.ui-progressbar .progress-20 {
  border: none; }

.show-container .progress-20,
.show-container .progress-40,
.show-container .progress-60,
.show-container .progress-80 {
  border-radius: 0;
  height: 7px; }

.show-title {
  position: relative;
  overflow: hidden;
  white-space: nowrap;
  font-size: 11px;
  margin: 4px 4px 0 4px; }

.show-title:after {
  content: "";
  pointer-events: none;
  position: absolute;
  width: 20px;
  height: 100%;
  top: 0;
  right: 0;
  background-image: -webkit-linear-gradient(right, #dfdacf, rgba(223, 218, 207, 0));
  background-image: -moz-linear-gradient(right, #dfdacf, rgba(223, 218, 207, 0));
  background-image: -ms-linear-gradient(right, #dfdacf, rgba(223, 218, 207, 0));
  background-image: -o-linear-gradient(right, #dfdacf, rgba(223, 218, 207, 0));
  background-image: linear-gradient(to left, #dfdacf, rgba(223, 218, 207, 0)); }

.show-date {
  position: relative;
  overflow: hidden;
  white-space: nowrap;
  font-size: 11px;
  margin: 0 4px 4px 4px; }

.show-date:after {
  content: "";
  pointer-events: none;
  position: absolute;
  width: 20px;
  height: 100%;
  top: 0;
  right: 0;
  background-image: -webkit-linear-gradient(right, #dfdacf, rgba(223, 218, 207, 0));
  background-image: -moz-linear-gradient(right, #dfdacf, rgba(223, 218, 207, 0));
  background-image: -ms-linear-gradient(right, #dfdacf, rgba(223, 218, 207, 0));
  background-image: -o-linear-gradient(right, #dfdacf, rgba(223, 218, 207, 0));
  background-image: linear-gradient(to left, #dfdacf, rgba(223, 218, 207, 0)); }

.show-table {
  text-align: center;
  vertical-align: middle;
  width: 33%; }

.show-add {
  font-size: 11px;
  text-align: left;
  display: block; }

.show-status {
  font-size: 11px;
  text-align: left;
  display: block; }

.show-network-image {
  width: 50px;
  height: auto; }

.show-dlstats {
  font-size: 11px;
  text-align: left;
  display: block;
  margin-left: 4px; }

.show-quality {
  font-size: 11px;
  text-align: right;
  display: block;
  margin-right: 4px; }

#sort-by {
  display: inline;
  list-style-type: none;
  padding: 0;
  margin-left: 5px; }

#sort-by ul, #sort-by li {
  display: inline;
  margin: 0;
  padding: 0; }

.posterview {
  margin: 0 auto; }

td.tvShow a {
  color: #000;
  text-decoration: none; }

td.tvShow a:hover {
  cursor: pointer;
  color: #428BCA; }

#popover-target label {
  margin: 0 5px;
  display: block; }

#popover-target input {
  margin-left: 5px; }

.popover {
  margin-left: -50px; }

/* =======================================================================
    add_shows.mako
    ========================================================================== */
#addShowPortal {
  width: 700px;
  padding: 10px 0;
  margin-right: auto;
  margin-left: auto; }

#addShowPortal a {
  padding: 10px; }

div.button {
  display: table-cell;
  vertical-align: middle;
  padding-left: 10px; }

div.buttontext {
  display: table-cell;
  padding-left: 20px;
  text-align: left;
  white-space: normal; }

div.buttontext h3 {
  margin-top: 10px; }

div.buttontext p {
  font-size: 13px; }

.icon-addnewshow {
  background-image: url("../images/addshows/add-new32-black.png");
  width: 32px;
  height: 32px; }

.icon-addtrendingshow {
  background-image: url("../images/addshows/add-trending32-black.png");
  width: 32px;
  height: 32px; }

.icon-addrecommendedshow {
  background-image: url("../images/addshows/add-trakt32-black.png");
  width: 32px;
  height: 32px; }

.icon-addexistingshow {
  background-image: url("../images/addshows/add-existing32-black.png");
  width: 32px;
  height: 32px; }

/* =======================================================================
    new_show.mako
    ========================================================================== */
#addShowForm, #recommendedShowsForm {
  font-size: 13px; }

#displayText {
  padding: 8px;
  overflow: hidden;
  font-size: 14px;
  background-color: #efefef;
  border: 1px solid #dfdede; }

#searchResults input[type="radio"] {
  vertical-align: -2px; }

/* =======================================================================
    add_existing_shows.mako
    ========================================================================== */
.existingtabs {
  padding: 1em 1.4em; }

ul#rootDirStaticList {
  margin-right: auto;
  margin-left: auto;
  text-align: left;
  padding-left: 0; }

ul#rootDirStaticList li {
  padding: 4px 5px 4px 5px;
  margin: 2px;
  list-style: none outside none;
  background: url("../images/ui-bg_highlight-soft_75_ffe45c_1x100.png") repeat-x scroll 50% 50% #EFEFEF; }

ul#rootDirStaticList li label {
  margin-top: 5px;
  margin-bottom: 5px;
  cursor: pointer; }

ul#rootDirStaticList li input[type="checkbox"] {
  vertical-align: -3px; }

/* =======================================================================
    trending_shows.mako
    ========================================================================== */
.traktShowTitleIcons {
  float: right;
  padding-right: 4px;
  padding-bottom: 4px; }

.traktContainer p {
  padding-top: 2px; }

.traktContainer p img {
  position: relative;
  top: -2px; }

.traktContainer p, .traktContainer i {
  white-space: nowrap;
  font-size: 12px;
  overflow: hidden;
  /*	text-shadow: 1px 1px 0px #000;*/
  padding-left: 4px;
  margin: 0; }

.traktContainer {
  margin: 12px;
  width: 188px;
  background-color: #DFDACF;
  border: 1px solid #111;
  border-radius: 6px; }

.trakt-image {
  overflow: hidden;
  height: 273px;
  width: 186px;
  border-top-left-radius: 5px;
  border-top-right-radius: 5px;
  border-bottom: 1px solid #111; }

/* =======================================================================
    postprocess.mako
    ========================================================================== */
#postProcess {
  width: 800px;
  padding-top: 10px;
  margin-right: auto;
  margin-left: auto; }

/* =======================================================================
    index.mako
    ========================================================================== */
#posterCol {
  float: left;
  margin-right: 10px;
  margin-bottom: 20px; }

#showCol {
  overflow: hidden;
  margin-bottom: 20px; }

#prevShow,
#nextShow,
#topcontrol {
  -webkit-filter: grayscale(100%);
  filter: grayscale(100%);
  filter: gray;
  /* IE6-9 */
  filter: url("data:image/svg+xml;utf8,<svg version='1.1' xmlns='http://www.w3.org/2000/svg' height='0'><filter id='greyscale'><feColorMatrix type='matrix' values='0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0 0 0 1 0' /></filter></svg>#greyscale"); }

h1.title {
  padding-bottom: 12px;
  margin-bottom: 15px;
  line-height: 30px;
  text-align: left;
  text-rendering: optimizelegibility;
  border-bottom: 1px solid #888; }

#showinfo {
  cursor: default;
  display: inline-block;
  position: relative;
  top: -3px; }

.imdbstars {
  display: inline-block;
  vertical-align: top;
  cursor: help;
  margin-top: 4px; }

.imdbstars, .imdbstars > * {
  height: 12px;
  background: url("../images/rating.png") 0 -12px repeat-x;
  width: 120px;
  display: inline-block;
  vertical-align: top; }

.imdbstars > * {
  background-position: 0 0;
  max-width: 120px; }

ul.tags {
  list-style-type: none;
  margin-left: -40px; }

ul.tags li {
  margin-right: 4px;
  margin-bottom: 4px;
  padding: 3px 4px 3px 25px;
  background: url("../images/tag.png") no-repeat scroll 5px 4px #555;
  border-radius: 3px;
  border: 1px solid #111;
  color: #FFF;
  font: 14px/18px roboto, "Open Sans", "Helvetica Neue", Helvetica, Arial, Geneva, sans-serif;
  text-shadow: 0 1px rgba(0, 0, 0, 0.8);
  float: left; }

ul.tags li a {
  color: #FFF; }

#summary {
  padding: 10px;
  background-color: #efefef;
  border: 1px solid #dfdede;
  width: 100%;
  height: 250px;
  overflow: auto;
  cursor: default;
  opacity: 0.8; }

#checkboxControls span {
  padding: 5px;
  font-size: 13px;
  font-weight: bold;
  border-radius: 5px; }

#checkboxControls label {
  white-space: nowrap;
  display: inline-block; }

#checkboxControls input[type="checkbox"] {
  vertical-align: -2px; }

.unaired {
  background-color: #f5f1e4; }

.skipped {
  background-color: #bedeed; }

.good {
  background-color: #c3e3c8; }

.qual {
  background-color: #ffda8a; }

.wanted {
  background-color: #ffb0b0; }

.snatched {
  background-color: #ebc1ea; }

span.unaired {
  color: #584b20;
  border: 1px solid #584b20; }

span.skipped {
  color: #1d5068;
  border: 1px solid #1d5068; }

span.good {
  color: #295730;
  border: 1px solid #295730; }

span.qual {
  color: #765100;
  border: 1px solid #765100; }

span.wanted {
  color: #890000;
  border: 1px solid #890000; }

span.snatched {
  color: #652164;
  border: 1px solid #652164; }

span.unaired b,
span.skipped b,
span.good b,
span.qual b,
span.wanted b,
span.snatched b {
  color: #000000;
  font-weight: 800; }

.plotInfo {
  cursor: help;
  float: right;
  position: relative;
  top: 2px; }

.plotInfoNone {
  cursor: help;
  float: right;
  position: relative;
  top: 2px;
  opacity: 0.4; }

.displayShowTable {
  table-layout: auto;
  width: 100%;
  border-collapse: collapse;
  border-spacing: 0;
  text-align: center;
  border: none;
  empty-cells: show;
  color: #000 !important; }

.displayShowTable .display_show {
  clear: both;
  opacity: 0.8; }

.displayShowTable th.row-seasonheader {
  border: none !important;
  background-color: #FFFFFF;
  color: #fff !important;
  padding-top: 15px !important;
  text-align: left !important; }

.displayShowTable td.col-search {
  text-align: center; }

.sickrageTable {
  table-layout: auto;
  width: 100%;
  border-collapse: collapse;
  border-spacing: 0;
  text-align: center;
  border: none;
  empty-cells: show; }

.sickrageTable.display_show {
  clear: both; }

.sickrageTable th {
  color: #fff;
  text-align: center;
  background-color: #333;
  white-space: nowrap; }

.sickrageTable th,
.sickrageTable td {
  border-top: 1px solid #fff;
  border-left: 1px solid #fff;
  padding: 4px; }

th.row-seasonheader {
  border: none;
  background-color: #fff;
  color: #000;
  padding-top: 15px;
  text-align: left; }

tr.seasonheader {
  padding-bottom: 5px;
  padding-top: 10px;
  text-align: left;
  border: none;
  color: #fff; }

th.col-checkbox,
td.col-checkbox {
  width: 30px;
  border-left: none;
  text-align: center; }

th.col-checkbox input[type="checkbox"],
td.col-checkbox input[type="checkbox"] {
  vertical-align: -2px; }

th.col-metadata,
td.col-metadata {
  width: 28px; }

th.col-ep,
td.col-ep {
  width: 50px;
  white-space: nowrap; }

th.col-airdate,
td.col-airdate {
  width: 86px;
  white-space: nowrap; }

th.col-name,
td.col-name {
  min-width: 100px; }

td.col-name {
  text-align: left; }

th.col-subtitles,
td.col-subtitles {
  width: 150px;
  text-align: center; }

th.col-status,
td.col-status {
  width: 230px;
  text-align: center; }

th.col-quality,
td.col-quality {
  width: 110px; }

th.col-legend,
td.col-legend {
  width: 80px; }

th.col-search,
td.col-search {
  width: 46px; }

.showLegend {
  padding-right: 6px;
  padding-bottom: 1px;
  width: 150px; }

.input-scene {
  height: 20px;
  line-height: 1.5;
  border-radius: 3px; }

#editShow {
  width: 700px;
  padding-top: 10px;
  margin-right: auto;
  margin-left: auto; }

/* =======================================================================
schedule.mako
========================================================================== */
.key {
  margin-bottom: 20px; }

.listing-key {
  padding: 5px;
  font-size: 13px;
  font-weight: bold;
  border-radius: 5px; }

.listing-default {
  background-color: #f5f1e4; }

.listing-current {
  background-color: #dfd; }

.listing-overdue {
  background-color: #fdd; }

.listing-toofar {
  background-color: #bedeed; }

span.listing-default {
  color: #826f30;
  border: 1px solid #826f30; }

span.listing-current {
  color: #295730;
  border: 1px solid #295730; }

span.listing-overdue {
  color: #890000;
  border: 1px solid #890000; }

span.listing-toofar {
  color: #1d5068;
  border: 1px solid #1d5068; }

h2.day, h2.network {
  margin: 10px 0;
  font-size: 24px;
  line-height: 36px;
  font-weight: bold;
  letter-spacing: 1px;
  color: #FFF;
  text-align: center;
  text-shadow: -1px -1px 0 rgba(0, 0, 0, 0.3);
  background-color: #333; }

.tvshowDiv {
  display: block;
  clear: both;
  border: 1px solid #CCCCCC;
  margin: auto;
  padding: 0;
  text-align: left;
  width: 100%;
  border-radius: 5px;
  background: #FFFFFF;
  cursor: default;
  overflow: hidden; }

.tvshowDiv a:hover {
  color: #428BCA; }

.tvshowDiv a, .tvshowDiv a:link, .tvshowDiv a:visited, .tvshowDiv a:hover {
  text-decoration: none;
  background: none; }

.tvshowTitle a {
  color: #000000;
  float: left;
  line-height: 1.4em;
  font-size: 1.4em;
  text-shadow: -1px -1px 0 #FFF; }

.tvshowTitleIcons {
  float: right;
  padding: 3px 5px; }

.tvshowDiv td {
  padding: 5px 10px; }

.tvshowDiv td.next_episode {
  width: 100%;
  height: 90%;
  border-bottom: 1px solid #ccc;
  vertical-align: top;
  color: #000; }

.bannerThumb {
  vertical-align: top;
  height: auto;
  width: 100%;
  border-bottom: 1px solid #ccc; }

.posterThumb {
  vertical-align: top;
  height: auto;
  width: 180px;
  border-right: 1px solid #ccc; }

.ep_listing {
  width: auto;
  border: 1px solid #ccc;
  margin-bottom: 10px;
  padding: 10px; }

.ep_summary {
  margin-left: 5px;
  font-style: italic; }

.ep_summaryTrigger {
  cursor: pointer;
  vertical-align: middle; }

.ep_summaryTriggerNone {
  opacity: 0.4;
  vertical-align: middle; }

.calendarWrapper {
  width: 1000px;
  margin: 0 auto;
  padding: 0 3px; }

.calendarTable {
  float: left;
  width: 142px !important;
  white-space: nowrap;
  table-layout: fixed !important; }

.calendarShow {
  padding: 0 !important; }

.calendarShow .poster {
  padding-bottom: 2px; }

.calendarShow .poster img {
  width: 142px;
  height: auto; }

.calendarShow .text {
  padding: 0 5px 10px 5px; }

.calendarShow .text .airtime,
.calendarShow .text .episode-title {
  overflow: hidden;
  text-overflow: ellipsis;
  display: block;
  font-size: 11px; }

.calendarShow .show-status {
  padding: 5px 10px 10px;
  text-align: center; }

/* =======================================================================
    config*.mako
    ========================================================================== */
#config div.field-pair {
  padding: 12px 0; }

#config div.field-pair select,
#config div.field-pair input {
  margin-right: 6px; }

#config div.field-pair input {
  float: left; }

#config .nocheck, #config div #customQuality {
  padding-left: 20px; }

#config span.component-title {
  float: left;
  width: 172px;
  margin-right: 10px;
  font-size: 13px;
  font-weight: bold; }

#config span.component-desc {
  font-size: 12px;
  font-weight: normal;
  display: block;
  width: 475px;
  margin-left: 182px; }

#config label.space-right {
  margin-right: 10px; }

#config .metadataDiv {
  display: none; }

.tab-pane-save {
  float: right;
  padding-top: 10px; }

select .selected {
  font-weight: 700; }

.jumbo {
  font-size: 15px !important;
  line-height: 24px; }

.testNotification {
  padding: 5px;
  margin-bottom: 10px;
  line-height: 20px;
  border: 1px dotted #CCC; }

#providerOrderList {
  width: 250px;
  padding-left: 20px;
  list-style-type: none; }

#provider_order_list,
#service_order_list {
  width: 250px;
  padding-left: 20px;
  list-style-type: none; }

#provider_order_list li,
#service_order_list li {
  padding: 5px;
  margin: 5px 0;
  font-size: 14px; }

#provider_order_list .ui-state-default.torrent-provider {
  background-color: #FFFFFF !important; }

#provider_order_list .ui-state-default.nzb-provider {
  background-color: #DDD !important; }

#provider_order_list input,
#service_order_list input {
  margin: 0 2px; }

#config .tip_scale label span.component-title {
  width: 85px !important;
  font-size: 12px !important;
  margin-top: 2px !important; }

#config .tip_scale label span.component-desc {
  margin-left: 120px !important;
  width: 220px !important; }

.infoTableHeader,
.infoTableCell {
  padding: 5px; }

.infoTableSeperator {
  border-top: 1px dotted #666666; }

/* =======================================================================
postprocessing.mako
========================================================================== */
#config div.example {
  padding: 10px;
  background-color: #efefef; }

.Key {
  width: 100%;
  padding: 6px;
  font-size: 13px;
  background-color: #f4f4f4;
  border: 1px solid #ccc;
  border-collapse: collapse;
  border-spacing: 0; }

.Key th, .tableHeader {
  padding: 3px 9px;
  margin: 0;
  color: #fff;
  text-align: center;
  background: none repeat scroll 0 0 #666; }

.Key td {
  padding: 1px 5px !important; }

.Key tr {
  border-bottom: 1px solid #ccc; }

.Key tr.even {
  background-color: #dfdede; }

.legend {
  position: relative;
  top: 2px; }

/* =======================================================================
notifications.mako
========================================================================== */
div.metadata_options {
  padding: 7px;
  overflow: auto;
  background: #F5F1E4;
  border: 1px solid #CCCCCC; }

div.metadata_options label:hover {
  color: #FFFFFF;
  background-color: #57442B;
  cursor: pointer; }

div.metadata_options label {
  display: block;
  padding-left: 7px;
  line-height: 20px;
  color: #003366; }

div.metadata_example {
  padding: 8px; }

div.metadata_example label {
  display: block;
  line-height: 21px;
  color: #000000;
  cursor: pointer; }

div.metadataDiv .disabled {
  color: #CCCCCC; }

.notifier-icon {
  float: left;
  margin: 6px 4px 0 0; }

.warning {
  border-color: #F89406;
  background: url("../images/warning16.png") no-repeat right 5px center #fff; }

/* =======================================================================
manage*.mako
========================================================================== */
.manageTable th {
  white-space: normal;
  line-height: 24px; }

.manageTable td.tableright {
  text-align: left; }

td.tableright {
  text-align: center; }

.optionWrapper {
  width: 450px;
  margin-left: auto;
  margin-right: auto;
  padding: 6px 12px; }

.optionWrapper span.selectTitle {
  float: left;
  text-align: left;
  vertical-align: middle;
  width: 225px;
  padding: 6px 0; }

.optionWrapper div.selectChoices {
  float: left;
  width: 175px;
  margin-left: 25px; }

.optionWrapper br {
  clear: both; }

.manageCustom {
  text-align: center;
  padding: 6px;
  margin-left: 25px; }

.separator {
  font-size: 90%;
  color: #333333; }

a.whitelink {
  color: #fff; }

/* =======================================================================
Global
========================================================================== */
span.path {
  padding: 3px 6px;
  color: #8b0000;
  background-color: #f5f1e4; }

.align-left {
  text-align: left; }

.h2footer {
  margin: -45px 0 8px;
  line-height: 18px; }

span.quality {
  font: 12px/13px "Open Sans", verdana, sans-serif;
  background-image: -webkit-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, transparent 50%, rgba(0, 0, 0, 0.25));
  background-image: -moz-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, transparent 50%, rgba(0, 0, 0, 0.25));
  background-image: -o-linear-gradient(top, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, transparent 50%, rgba(0, 0, 0, 0.25));
  background-image: linear-gradient(to bottom, rgba(255, 255, 255, 0.08), rgba(255, 255, 255, 0) 50%, transparent 50%, rgba(0, 0, 0, 0.25));
  -webkit-box-shadow: inset 0 1px rgba(255, 255, 255, 0.1), inset 0 -1px 3px rgba(0, 0, 0, 0.3), inset 0 0 0 1px rgba(255, 255, 255, 0.08), 0 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: inset 0 1px rgba(255, 255, 255, 0.1), inset 0 -1px 3px rgba(0, 0, 0, 0.3), inset 0 0 0 1px rgba(255, 255, 255, 0.08), 0 1px 2px rgba(0, 0, 0, 0.15);
  text-shadow: 0 1px rgba(0, 0, 0, 0.8);
  color: #FFFFFF;
  display: inline-block;
  padding: 2px 4px;
  text-align: center;
  vertical-align: baseline;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  white-space: nowrap; }

span.any-hd {
  background: #2672b6 repeating-linear-gradient(-45deg, #2672b6, #2672b6 10px, #5b990d 10px, #5b990d 20px); }

span.Custom {
  background-color: #621993; }

span.HD {
  background-color: #2672B6; }

span.HDTV {
  background-color: #2672B6; }

span.HD720p {
  background-color: #5b990d; }

span.HD1080p {
  background-color: #2672B6; }

span.UHD-4K {
  background-color: #7500FF; }

span.UHD-8K {
  background-color: #410077; }

span.RawHD {
  background-color: #cd7300; }

span.RawHDTV {
  background-color: #cd7300; }

span.SD {
  background-color: #BE2625; }

span.SDTV {
  background-color: #BE2625; }

span.SDDVD {
  background-color: #BE2625; }

span.Any {
  background-color: #666; }

span.Unknown {
  background-color: #999; }

span.Proper {
  background-color: #3F7F00; }

span.false {
  color: #993333;
  /* red */ }

span.true {
  color: #669966;
  /* green */ }

option.flag {
  padding-left: 35px;
  background-repeat: no-repeat;
  background-position: 10px 50%; }

/* Anime section for editShow */
.bwlWrapper {
  height: auto;
  margin: 0 auto; }

#Anime {
  clear: both;
  overflow-x: hidden;
  overflow-y: hidden;
  font-size: 14px; }

#Anime div.tab-pane-desc {
  float: left;
  width: 165px; }

#Anime div.tab-pane-desc p {
  margin-bottom: 0.4em;
  margin-left: 0;
  margin-right: 0;
  margin-top: 0.4em;
  width: 95%; }

div.blackwhitelist {
  float: left;
  text-align: center; }

div.blackwhitelist input {
  margin: 5px 0; }

div.blackwhitelist.pool select {
  width: 230px; }

div.blackwhitelist.white select, div.blackwhitelist.black select {
  width: 150px; }

div.blackwhitelist span {
  display: block;
  text-align: center; }

div.blackwhitelist.anidb, div.blackwhitelist.manual {
  margin: 7px 0; }

/* =======================================================================
bootstrap Overrides
========================================================================== */
body {
  padding-top: 4%;
  overflow-y: scroll;
  font-family: roboto, "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
  color: #000; }

html * {
  outline: 0 !important; }

input[type="radio"] {
  margin: 2px 0 0;
  line-height: normal; }

input, textarea, select, .uneditable-input {
  width: auto;
  color: #000; }

.container-fluid {
  margin-left: 10px;
  margin-right: 10px; }

.navbar-brand {
  padding: 0; }

.navbar-default {
  background-color: #333333;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#555555', endColorstr='#333333');
  background: -webkit-gradient(linear, left top, left bottom, from(#555555), to(#333333));
  background: -moz-linear-gradient(top, #555555, #333333);
  border-color: #3E3F3A; }

.navbar-default .navbar-collapse,
.navbar-default .navbar-form {
  border-color: #3E3F3A; }

.navbar-default .navbar-nav > li > a:hover,
.navbar-default .navbar-nav > li > a:focus {
  background-color: #333333; }

.dropdown-menu > li > a:hover, .dropdown-menu > li > a:focus {
  color: #FFFFFF;
  text-decoration: none;
  background-color: #333333; }

.navbar-default .navbar-brand {
  color: #FFFFFF; }

.navbar-default .navbar-brand:hover,
.navbar-default .navbar-brand:focus {
  color: #FFFFFF;
  background-color: transparent; }

.navbar-default .navbar-text {
  color: #DDDDDD; }

.navbar-default .navbar-nav > li > a {
  color: #DDDDDD; }

.navbar-default .navbar-nav > li > a:hover,
.navbar-default .navbar-nav > li > a:focus {
  color: #FFFFFF; }

.navbar-default .navbar-nav > .active > a,
.navbar-default .navbar-nav > .active > a:hover,
.navbar-default .navbar-nav > .active > a:focus {
  color: #FFFFFF;
  background-color: #333333; }

.navbar-default .navbar-nav > .disabled > a,
.navbar-default .navbar-nav > .disabled > a:hover,
.navbar-default .navbar-nav > .disabled > a:focus {
  color: #CCCCCC;
  background-color: transparent; }

.navbar-default .navbar-toggle {
  border-color: #CCCCCC; }

.navbar-default .navbar-toggle:hover,
.navbar-default .navbar-toggle:focus {
  background-color: #333333; }

.navbar-default .navbar-toggle .icon-bar {
  background-color: #333333; }

.navbar-default .navbar-toggle:hover .icon-bar,
.navbar-default .navbar-toggle:focus .icon-bar {
  background-color: #15528F; }

.navbar-default .navbar-nav > .open > a,
.navbar-default .navbar-nav > .open > a:hover,
.navbar-default .navbar-nav > .open > a:focus {
  background-color: #333333;
  color: #FFFFFF; }

.navbar-default .navbar-nav > li.navbar-split > a {
  display: inline-block;
  float: left; }

.navbar-default .navbar-nav > li.navbar-split > a:first-child {
  padding-right: 8px; }

.navbar-default .navbar-nav > li.navbar-split > a:nth-child(2) {
  padding-left: 8px; }

@media (max-width: 767px) {
  .navbar-default .navbar-nav .open .dropdown-menu > li > a {
    color: #DDDDDD; }
  .navbar-default .navbar-nav .open .dropdown-menu > li > a:hover,
  .navbar-default .navbar-nav .open .dropdown-menu > li > a:focus {
    color: #FFFFFF;
    background-color: transparent; }
  .navbar-default .navbar-nav .open .dropdown-menu > .active > a,
  .navbar-default .navbar-nav .open .dropdown-menu > .active > a:hover,
  .navbar-default .navbar-nav .open .dropdown-menu > .active > a:focus {
    color: #FFFFFF;
    background-color: #333333; }
  .navbar-default .navbar-nav .open .dropdown-menu > .disabled > a,
  .navbar-default .navbar-nav .open .dropdown-menu > .disabled > a:hover,
  .navbar-default .navbar-nav .open .dropdown-menu > .disabled > a:focus {
    color: #CCCCCC;
    background-color: transparent; } }

.navbar-default .navbar-link {
  color: #DDDDDD; }

.navbar-default .navbar-link:hover {
  color: #FFFFFF; }

.navbar-default .btn-link {
  color: #98978B; }

.navbar-default .btn-link:hover,
.navbar-default .btn-link:focus {
  color: #DDDDDD; }

.navbar-default .btn-link[disabled]:hover,
fieldset[disabled] .navbar-default .btn-link:hover,
.navbar-default .btn-link[disabled]:focus,
fieldset[disabled] .navbar-default .btn-link:focus {
  color: #CCCCCC; }

.dropdown-menu > li > a:hover, .dropdown-menu > li > a:focus {
  color: #262626;
  text-decoration: none;
  background-color: #F5F5F5; }

.dropdown-menu > li > a {
  padding: 4px 36px 4px 20px; }

.dropdown-menu {
  background-color: #F5F1E4;
  border: 1px solid rgba(0, 0, 0, 0.15);
  box-shadow: 0 6px 12px rgba(0, 0, 0, 0.176); }

.form-control {
  color: #000000; }

.form-control-inline {
  min-width: 0;
  width: auto;
  display: inline; }

.btn {
  display: inline-block;
  *display: inline;
  padding: 4px 10px 4px;
  margin-bottom: 0;
  *margin-left: .3em;
  font-size: 12px;
  line-height: 16px;
  *line-height: 20px;
  color: #333333;
  text-align: center;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
  vertical-align: middle;
  cursor: pointer;
  background-color: #f5f5f5;
  *background-color: #e6e6e6;
  background-image: -ms-linear-gradient(top, #ffffff, #e6e6e6);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), to(#e6e6e6));
  background-image: -webkit-linear-gradient(top, #ffffff, #e6e6e6);
  background-image: -o-linear-gradient(top, #ffffff, #e6e6e6);
  background-image: linear-gradient(top, #ffffff, #e6e6e6);
  background-image: -moz-linear-gradient(top, #ffffff, #e6e6e6);
  background-repeat: repeat-x;
  border: 1px solid #cccccc;
  *border: 0;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  border-color: #e6e6e6 #e6e6e6 #bfbfbf;
  border-bottom-color: #b3b3b3;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false);
  *zoom: 1;
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05); }

.btn:hover,
.btn:active,
.btn.active,
.btn.disabled,
.btn[disabled] {
  background-color: #e6e6e6;
  *background-color: #d9d9d9; }

.btn:active,
.btn.active {
  background-color: #cccccc \9; }

.btn:first-child {
  *margin-left: 0; }

.btn:hover {
  color: #333333;
  text-decoration: none;
  background-color: #e6e6e6;
  *background-color: #d9d9d9;
  background-position: 0 -15px;
  -webkit-transition: background-position 0.1s linear;
  -moz-transition: background-position 0.1s linear;
  -ms-transition: background-position 0.1s linear;
  -o-transition: background-position 0.1s linear;
  transition: background-position 0.1s linear; }

.btn:focus {
  outline: thin dotted #333;
  outline: 5px auto -webkit-focus-ring-color;
  outline-offset: -2px; }

.btn.active,
.btn:active {
  background-color: #e6e6e6;
  background-color: #d9d9d9 \9;
  background-image: none;
  outline: 0;
  -webkit-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05); }

.btn.disabled,
.btn[disabled] {
  cursor: default;
  background-color: #e6e6e6;
  background-image: none;
  opacity: 0.65;
  filter: alpha(opacity=65);
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none; }

.btn-large {
  padding: 9px 14px;
  font-size: 15px;
  line-height: normal;
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  border-radius: 5px; }

.btn-large [class^="icon-"] {
  margin-top: 1px; }

.btn-small {
  padding: 5px 9px;
  font-size: 11px;
  line-height: 16px; }

.btn-small [class^="icon-"] {
  margin-top: -1px; }

.btn-mini {
  padding: 2px 6px;
  font-size: 11px;
  line-height: 14px; }

.btn-primary,
.btn-primary:hover,
.btn-warning,
.btn-warning:hover,
.btn-danger,
.btn-danger:hover,
.btn-success,
.btn-success:hover,
.btn-info,
.btn-info:hover,
.btn-inverse,
.btn-inverse:hover {
  color: #ffffff;
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25); }

.btn-primary.active,
.btn-warning.active,
.btn-danger.active,
.btn-success.active,
.btn-info.active,
.btn-inverse.active {
  color: rgba(255, 255, 255, 0.75); }

.btn-primary {
  background-color: #0074cc;
  *background-color: #0055cc;
  background-image: -ms-linear-gradient(top, #0088cc, #0055cc);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#0088cc), to(#0055cc));
  background-image: -webkit-linear-gradient(top, #0088cc, #0055cc);
  background-image: -o-linear-gradient(top, #0088cc, #0055cc);
  background-image: -moz-linear-gradient(top, #0088cc, #0055cc);
  background-image: linear-gradient(top, #0088cc, #0055cc);
  background-repeat: repeat-x;
  border-color: #0055cc #0055cc #003580;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#0088cc', endColorstr='#0055cc', GradientType=0);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false); }

.btn-primary:hover,
.btn-primary:active,
.btn-primary.active,
.btn-primary.disabled,
.btn-primary[disabled] {
  background-color: #0055cc;
  *background-color: #004ab3; }

.btn-primary:active,
.btn-primary.active {
  background-color: #004099 \9; }

.btn-warning {
  background-color: #faa732;
  *background-color: #f89406;
  background-image: -ms-linear-gradient(top, #fbb450, #f89406);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#fbb450), to(#f89406));
  background-image: -webkit-linear-gradient(top, #fbb450, #f89406);
  background-image: -o-linear-gradient(top, #fbb450, #f89406);
  background-image: -moz-linear-gradient(top, #fbb450, #f89406);
  background-image: linear-gradient(top, #fbb450, #f89406);
  background-repeat: repeat-x;
  border-color: #f89406 #f89406 #ad6704;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#fbb450', endColorstr='#f89406', GradientType=0);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false); }

.btn-warning:hover,
.btn-warning:active,
.btn-warning.active,
.btn-warning.disabled,
.btn-warning[disabled] {
  background-color: #f89406;
  *background-color: #df8505; }

.btn-warning:active,
.btn-warning.active {
  background-color: #c67605 \9; }

.btn-danger {
  background-color: #da4f49;
  *background-color: #bd362f;
  background-image: -ms-linear-gradient(top, #ee5f5b, #bd362f);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ee5f5b), to(#bd362f));
  background-image: -webkit-linear-gradient(top, #ee5f5b, #bd362f);
  background-image: -o-linear-gradient(top, #ee5f5b, #bd362f);
  background-image: -moz-linear-gradient(top, #ee5f5b, #bd362f);
  background-image: linear-gradient(top, #ee5f5b, #bd362f);
  background-repeat: repeat-x;
  border-color: #bd362f #bd362f #802420;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#bd362f', GradientType=0);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false); }

.btn-danger:hover,
.btn-danger:active,
.btn-danger.active,
.btn-danger.disabled,
.btn-danger[disabled] {
  background-color: #bd362f;
  *background-color: #a9302a; }

.btn-danger:active,
.btn-danger.active {
  background-color: #942a25 \9; }

.btn-success {
  background-color: #5bb75b;
  *background-color: #51a351;
  background-image: -ms-linear-gradient(top, #62c462, #51a351);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#62c462), to(#51a351));
  background-image: -webkit-linear-gradient(top, #62c462, #51a351);
  background-image: -o-linear-gradient(top, #62c462, #51a351);
  background-image: -moz-linear-gradient(top, #62c462, #51a351);
  background-image: linear-gradient(top, #62c462, #51a351);
  background-repeat: repeat-x;
  border-color: #51a351 #51a351 #387038;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#62c462', endColorstr='#51a351', GradientType=0);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false); }

.btn-success:hover,
.btn-success:active,
.btn-success.active,
.btn-success.disabled,
.btn-success[disabled] {
  background-color: #51a351;
  *background-color: #499249; }

.btn-success:active,
.btn-success.active {
  background-color: #408140 \9; }

.btn-info {
  background-color: #49afcd;
  *background-color: #2f96b4;
  background-image: -ms-linear-gradient(top, #5bc0de, #2f96b4);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#5bc0de), to(#2f96b4));
  background-image: -webkit-linear-gradient(top, #5bc0de, #2f96b4);
  background-image: -o-linear-gradient(top, #5bc0de, #2f96b4);
  background-image: -moz-linear-gradient(top, #5bc0de, #2f96b4);
  background-image: linear-gradient(top, #5bc0de, #2f96b4);
  background-repeat: repeat-x;
  border-color: #2f96b4 #2f96b4 #1f6377;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#5bc0de', endColorstr='#2f96b4', GradientType=0);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false); }

.btn-info:hover,
.btn-info:active,
.btn-info.active,
.btn-info.disabled,
.btn-info[disabled] {
  background-color: #2f96b4;
  *background-color: #2a85a0; }

.btn-info:active,
.btn-info.active {
  background-color: #24748c \9; }

.btn-inverse {
  background-color: #414141;
  *background-color: #222222;
  background-image: -ms-linear-gradient(top, #555555, #222222);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#555555), to(#222222));
  background-image: -webkit-linear-gradient(top, #555555, #222222);
  background-image: -o-linear-gradient(top, #555555, #222222);
  background-image: -moz-linear-gradient(top, #555555, #222222);
  background-image: linear-gradient(top, #555555, #222222);
  background-repeat: repeat-x;
  border-color: #222222 #222222 #000000;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#555555', endColorstr='#222222', GradientType=0);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false); }

.btn-inverse:hover,
.btn-inverse:active,
.btn-inverse.active,
.btn-inverse.disabled,
.btn-inverse[disabled] {
  background-color: #222222;
  *background-color: #151515; }

.btn-inverse:active,
.btn-inverse.active {
  background-color: #080808 \9; }

.btn-xs {
  padding: 1px 5px;
  font-size: 12px;
  line-height: 1.5;
  border-radius: 3px; }

.navbar .nav > li > .dropdown-menu:after {
  position: absolute;
  top: -6px;
  right: 12px;
  display: inline-block;
  border-right: 6px solid transparent;
  border-bottom: 6px solid #F5F1E4;
  border-left: 6px solid transparent;
  content: ""; }

label {
  font-weight: normal; }

pre {
  border: 1px solid; }

.alert .glyphicon {
  display: table-cell; }

.alert div,
.alert span {
  padding-left: 5px;
  display: table-cell; }

/* =======================================================================
input sizing (for config pages)
========================================================================== */
.btn-inline {
  margin-top: -3px; }

.input75 {
  width: 75px;
  margin-top: -4px; }

.input100 {
  width: 100px;
  margin-top: -4px; }

.input150 {
  width: 150px;
  margin-top: -4px; }

.input200 {
  width: 200px;
  margin-top: -4px; }

.input250 {
  width: 250px;
  margin-top: -4px; }

.input300 {
  width: 300px;
  margin-top: -4px; }

.input350 {
  width: 350px;
  margin-top: -4px; }

/* =======================================================================
browser.css overrides
========================================================================== */
#fileBrowserDialog {
  overflow-y: auto; }

#fileBrowserDialog ul li a:hover {
  color: #00f;
  background: none; }

#fileBrowserDialog h2 {
  font-size: 20px; }

.ui-autocomplete {
  max-height: 180px;
  overflow-x: hidden;
  overflow-y: auto; }

/* IE6 hack since it doesn't support max-height */
* html .ui-autocomplete {
  height: 180px;
  padding-right: 20px; }

.ui-autocomplete .ui-menu-item .ui-state-focus {
  color: #fff;
  background: none;
  background-color: #0a246a; }

/* restore 1.8.x resize handle on dialog button pane */
.ui-dialog .ui-resizable-se {
  width: 14px;
  height: 14px;
  right: 3px;
  bottom: 3px;
  background-position: -80px -224px; }

/* =======================================================================
jquery-steps.css
========================================================================== */
.wizard .content {
  background: transparent;
  min-height: 100px; }

.wizard .content > .body {
  width: 100%;
  height: auto;
  padding: 15px;
  position: relative; }

.wizard > .content > section {
  text-align: left;
  color: #57442b; }

/* step 3 related */
#customQuality {
  display: block;
  padding: 10px 0;
  overflow: hidden;
  clear: both; }

#customQuality div.tab-pane-desc {
  float: left;
  width: 172px;
  margin-right: 10px; }

#customQuality div.tab-pane-desc p {
  width: 85%;
  margin: .8em 0;
  font-size: 1.13em;
  color: #666; }

/* =======================================================================
tablesorter.css
========================================================================== */
.tablesorter {
  width: 100%;
  margin-right: auto;
  margin-left: auto;
  color: #000000;
  text-align: left;
  background-color: #DDDDDD;
  opacity: 0.8;
  border-spacing: 0; }

.tablesorter th,
.tablesorter td {
  padding: 4px;
  border-top: #FFFFFF 1px solid;
  border-left: #FFFFFF 1px solid;
  vertical-align: middle; }

/* add extra border at right edge */
.tablesorter th:last-child,
.tablesorter td:last-child {
  border-right: #FFFFFF 1px solid; }

.tablesorter th {
  color: #FFFFFF;
  text-align: center;
  text-shadow: -1px -1px 0 rgba(0, 0, 0, 0.3);
  background-color: #333333;
  border-collapse: collapse;
  font-weight: normal; }

.tablesorter .tablesorter-header {
  padding: 4px 18px 4px 18px;
  cursor: pointer;
  background: inherit url(data:image/gif;base64,R0lGODlhFQAJAIAAAP///////yH5BAEAAAEALAAAAAAVAAkAAAIXjI+AywnaYnhUMoqt3gZXPmVg94yJVQAAOw==) no-repeat center right; }

.tablesorter thead .tablesorter-headerDesc {
  background: #555555 url(data:image/gif;base64,R0lGODlhFQAEAIAAAP///////yH5BAEAAAEALAAAAAAVAAQAAAINjB+gC+jP2ptn0WskLQA7) no-repeat center right; }

.tablesorter thead .tablesorter-headerAsc {
  background: #555555 url(data:image/gif;base64,R0lGODlhFQAEAIAAAP///////yH5BAEAAAEALAAAAAAVAAQAAAINjI8Bya2wnINUMopZAQA7) no-repeat center right; }

.tablesorter thead .sorter-false {
  background-image: none;
  padding: 4px;
  cursor: default; }

thead.tablesorter-stickyHeader {
  border-top: 2px solid #FFFFFF;
  border-bottom: 2px solid #FFFFFF; }

/* Zebra Widget - row alternating colors */
.tablesorter tr.odd, .sickbeardTable tr.odd {
  background-color: #F5F1E4; }

.tablesorter tr.even, .sickbeardTable tr.even {
  background-color: #DFDACF; }

/* filter widget */
.tablesorter .filtered {
  display: none; }

.tablesorter input.tablesorter-filter {
  width: 98%;
  height: auto;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box; }

.tablesorter tr.tablesorter-filter-row,
.tablesorter tr.tablesorter-filter-row td {
  text-align: center;
  background: #EEEEEE;
  border-bottom: 1px solid #DDDDDD; }

/* add extra border at left edge */
.tablesorter tr.tablesorter-filter-row td:first-child {
  border-left: #FFFFFF 1px solid; }

/* add extra border at right edge */
.tablesorter tr.tablesorter-filter-row td:last-child {
  border-right: #FFFFFF 1px solid; }

/* hidden filter row */
.tablesorter-filter-row.hideme td {
  /*** *********************************************** ***/
  /*** change this padding to modify the thickness     ***/
  /*** of the closed filter row (height = padding x 2) ***/
  padding: 2px;
  /*** *********************************************** ***/
  margin: 0;
  line-height: 0;
  cursor: pointer; }

.tablesorter-filter-row.hideme * {
  height: 1px;
  min-height: 0;
  border: 0;
  padding: 0;
  margin: 0;
  opacity: 0;
  filter: alpha(opacity=0); }

/* optional disabled input styling */
.tablesorter input.tablesorter-filter-row .disabled {
  display: none; }

.tablesorter-header-inner {
  padding: 0 2px;
  text-align: center; }

.tablesorter tfoot tr {
  color: #FFFFFF;
  text-align: center;
  text-shadow: -1px -1px 0 rgba(0, 0, 0, 0.3);
  background-color: #333333;
  border-collapse: collapse; }

.tablesorter tfoot a {
  color: #FFFFFF;
  text-decoration: none; }

/* =======================================================================
token-input.css
========================================================================== */
ul.token-input-list {
  overflow: hidden;
  height: auto !important;
  height: 1%;
  width: 273px;
  border: 1px solid #ccc;
  cursor: text;
  font-size: 10px;
  font-family: Verdana, Geneva, sans-serif;
  z-index: 999;
  margin: 0;
  padding: 0 0 1px 0;
  background-color: #fff;
  list-style-type: none;
  /*    clear: left; */
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
  border-bottom-left-radius: 3px;
  border-bottom-right-radius: 3px; }

ul.token-input-list li {
  list-style-type: none; }

ul.token-input-list li input {
  border: 0;
  padding: 3px 4px;
  background-color: white;
  /*    -webkit-appearance: caret; */ }

li.token-input-token {
  overflow: hidden;
  height: auto !important;
  height: 1%;
  margin: 3px;
  padding: 3px 5px 0 5px;
  background-color: #d0efa0;
  color: #000;
  font-weight: bold;
  cursor: default;
  display: block; }

li.token-input-token img {
  padding-top: 3px;
  padding-right: 4px;
  float: left; }

li.token-input-token input {
  padding-top: 2px !important;
  padding-right: 4px !important;
  float: left; }

li.token-input-token p {
  float: left;
  padding: 0;
  margin: 0;
  line-height: 2.0 !important; }

li.token-input-token span {
  float: right;
  color: #777;
  cursor: pointer; }

li.token-input-selected-token {
  background-color: #08844e;
  color: #fff; }

li.token-input-selected-token span {
  color: #bbb; }

li.token-input-input-token input {
  margin: 3px 3px 3px 3px !important; }

div.token-input-dropdown {
  position: absolute;
  width: 273px;
  overflow: hidden;
  border-left: 1px solid;
  border-right: 1px solid;
  border-bottom: 1px solid;
  cursor: default;
  font-size: 11px;
  font-family: Verdana, Geneva, sans-serif;
  z-index: 1; }

div.token-input-dropdown p {
  margin: 0;
  padding: 3px;
  font-weight: bold;
  color: #777; }

div.token-input-dropdown ul {
  margin: 0;
  padding: 0; }

div.token-input-dropdown ul li {
  background-color: #fff;
  padding: 3px;
  list-style-type: none; }

div.token-input-dropdown ul li.token-input-dropdown-item {
  background-color: #fafafa; }

div.token-input-dropdown ul li.token-input-dropdown-item2 {
  background-color: #fff; }

div.token-input-dropdown ul li em {
  font-weight: bold;
  font-style: normal; }

div.token-input-dropdown ul li.token-input-selected-dropdown-item {
  background-color: #6196c2; }

span.token-input-delete-token {
  margin: 0 1px; }

.red-text {
  color: #d33; }

.clear-left {
  clear: left; }

.nextline-block {
  display: block; }

.trakt-image {
  display: block;
  width: 100%;
  height: 100%;
  z-index: 0;
  background-image: url("../images/poster-dark.jpg"); }

/* =======================================================================
pnotify.css
========================================================================== */
.ui-pnotify-sticker {
  margin-top: -12px; }

.ui-pnotify-closer {
  margin-top: -12px;
  margin-right: -10px; }

/* =======================================================================
IMDB Popular
========================================================================== */
.popularShow {
  margin-bottom: 30px; }

.popularShow h3 {
  padding: 0;
  margin: 0;
  display: inline-block;
  margin-right: 30px; }

.popularShow .left {
  float: left;
  width: 100px; }

.popularShow .right {
  float: left;
  width: 600px; }

.popularShow .year {
  font-style: italic;
  opacity: 0.7; }

.popularShow .coverImage {
  width: 80%;
  padding-left: 20px;
  margin-top: 4px; }

.popularShow .rating {
  font-size: 90%;
  display: inline-block;
  margin-left: 0; }

.popularShow p {
  margin-bottom: 0; }

/* =======================================================================
File Browser
========================================================================== */
#fileBrowserDialog {
  max-height: 480px;
  overflow-y: auto; }

#fileBrowserDialog ul {
  margin: 0;
  padding: 0; }

#fileBrowserDialog ul li {
  margin: 2px 0;
  cursor: pointer;
  list-style-type: none; }

#fileBrowserDialog ul li a {
  display: block;
  padding: 4px 0; }

#fileBrowserDialog ul li a:hover {
  color: #00f;
  background: none; }

#fileBrowserDialog ul li a span.ui-icon {
  margin: 0 4px;
  float: left; }

/* jQuery-UI autocomplete overrides to make it look more like the old autocomplete */
.ui-autocomplete {
  max-height: 180px;
  overflow-y: auto;
  /* prevent horizontal scrollbar */
  overflow-x: hidden;
  /* add padding to account for vertical scrollbar */
  padding-right: 20px; }

* html .ui-autocomplete {
  height: 180px; }

.ui-menu .ui-menu-item {
  background-color: #eeeeee; }

.ui-menu .ui-menu-item-alternate {
  background-color: #ffffff; }

.ui-menu a.ui-state-hover {
  background: none;
  background-color: #0A246A;
  color: #ffffff; }

/* =======================================================================
Country Flags
========================================================================== */
.country-flag {
  width: 16px;
  height: 11px;
  background: url("../images/country-flags.png") no-repeat; }

.country-flag.flag-ad {
  background-position: -16px 0; }

.country-flag.flag-ae {
  background-position: -32px 0; }

.country-flag.flag-af {
  background-position: -48px 0; }

.country-flag.flag-ag {
  background-position: -64px 0; }

.country-flag.flag-ai {
  background-position: -80px 0; }

.country-flag.flag-al {
  background-position: -96px 0; }

.country-flag.flag-am {
  background-position: -112px 0; }

.country-flag.flag-an {
  background-position: -128px 0; }

.country-flag.flag-ao {
  background-position: -144px 0; }

.country-flag.flag-ar {
  background-position: -160px 0; }

.country-flag.flag-as {
  background-position: -176px 0; }

.country-flag.flag-at {
  background-position: -192px 0; }

.country-flag.flag-au {
  background-position: -208px 0; }

.country-flag.flag-aw {
  background-position: -224px 0; }

.country-flag.flag-az {
  background-position: -240px 0; }

.country-flag.flag-ba {
  background-position: 0 -11px; }

.country-flag.flag-bb {
  background-position: -16px -11px; }

.country-flag.flag-bd {
  background-position: -32px -11px; }

.country-flag.flag-be {
  background-position: -48px -11px; }

.country-flag.flag-bf {
  background-position: -64px -11px; }

.country-flag.flag-bg {
  background-position: -80px -11px; }

.country-flag.flag-bh {
  background-position: -96px -11px; }

.country-flag.flag-bi {
  background-position: -112px -11px; }

.country-flag.flag-bj {
  background-position: -128px -11px; }

.country-flag.flag-bm {
  background-position: -144px -11px; }

.country-flag.flag-bn {
  background-position: -160px -11px; }

.country-flag.flag-bo {
  background-position: -176px -11px; }

.country-flag.flag-br {
  background-position: -192px -11px; }

.country-flag.flag-bs {
  background-position: -208px -11px; }

.country-flag.flag-bt {
  background-position: -224px -11px; }

.country-flag.flag-bv {
  background-position: -240px -11px; }

.country-flag.flag-bw {
  background-position: 0 -22px; }

.country-flag.flag-by {
  background-position: -16px -22px; }

.country-flag.flag-bz {
  background-position: -32px -22px; }

.country-flag.flag-ca {
  background-position: -48px -22px; }

.country-flag.flag-catalonia {
  background-position: -64px -22px; }

.country-flag.flag-cd {
  background-position: -80px -22px; }

.country-flag.flag-cf {
  background-position: -96px -22px; }

.country-flag.flag-cg {
  background-position: -112px -22px; }

.country-flag.flag-ch {
  background-position: -128px -22px; }

.country-flag.flag-ci {
  background-position: -144px -22px; }

.country-flag.flag-ck {
  background-position: -160px -22px; }

.country-flag.flag-cl {
  background-position: -176px -22px; }

.country-flag.flag-cm {
  background-position: -192px -22px; }

.country-flag.flag-cn {
  background-position: -208px -22px; }

.country-flag.flag-co {
  background-position: -224px -22px; }

.country-flag.flag-cr {
  background-position: -240px -22px; }

.country-flag.flag-cu {
  background-position: 0 -33px; }

.country-flag.flag-cv {
  background-position: -16px -33px; }

.country-flag.flag-cw {
  background-position: -32px -33px; }

.country-flag.flag-cy {
  background-position: -48px -33px; }

.country-flag.flag-cz {
  background-position: -64px -33px; }

.country-flag.flag-de {
  background-position: -80px -33px; }

.country-flag.flag-dj {
  background-position: -96px -33px; }

.country-flag.flag-dk {
  background-position: -112px -33px; }

.country-flag.flag-dm {
  background-position: -128px -33px; }

.country-flag.flag-do {
  background-position: -144px -33px; }

.country-flag.flag-dz {
  background-position: -160px -33px; }

.country-flag.flag-ec {
  background-position: -176px -33px; }

.country-flag.flag-ee {
  background-position: -192px -33px; }

.country-flag.flag-eg {
  background-position: -208px -33px; }

.country-flag.flag-eh {
  background-position: -224px -33px; }

.country-flag.flag-england {
  background-position: -240px -33px; }

.country-flag.flag-er {
  background-position: 0 -44px; }

.country-flag.flag-es {
  background-position: -16px -44px; }

.country-flag.flag-et {
  background-position: -32px -44px; }

.country-flag.flag-eu {
  background-position: -48px -44px; }

.country-flag.flag-fi {
  background-position: -64px -44px; }

.country-flag.flag-fj {
  background-position: -80px -44px; }

.country-flag.flag-fk {
  background-position: -96px -44px; }

.country-flag.flag-fm {
  background-position: -112px -44px; }

.country-flag.flag-fo {
  background-position: -128px -44px; }

.country-flag.flag-fr {
  background-position: -144px -44px; }

.country-flag.flag-ga {
  background-position: -160px -44px; }

.country-flag.flag-gb {
  background-position: -176px -44px; }

.country-flag.flag-gd {
  background-position: -192px -44px; }

.country-flag.flag-ge {
  background-position: -208px -44px; }

.country-flag.flag-gf {
  background-position: -224px -44px; }

.country-flag.flag-gg {
  background-position: -240px -44px; }

.country-flag.flag-gh {
  background-position: 0 -55px; }

.country-flag.flag-gi {
  background-position: -16px -55px; }

.country-flag.flag-gl {
  background-position: -32px -55px; }

.country-flag.flag-gm {
  background-position: -48px -55px; }

.country-flag.flag-gn {
  background-position: -64px -55px; }

.country-flag.flag-gp {
  background-position: -80px -55px; }

.country-flag.flag-gq {
  background-position: -96px -55px; }

.country-flag.flag-gr {
  background-position: -112px -55px; }

.country-flag.flag-gs {
  background-position: -128px -55px; }

.country-flag.flag-gt {
  background-position: -144px -55px; }

.country-flag.flag-gu {
  background-position: -160px -55px; }

.country-flag.flag-gw {
  background-position: -176px -55px; }

.country-flag.flag-gy {
  background-position: -192px -55px; }

.country-flag.flag-hk {
  background-position: -208px -55px; }

.country-flag.flag-hm {
  background-position: -224px -55px; }

.country-flag.flag-hn {
  background-position: -240px -55px; }

.country-flag.flag-hr {
  background-position: 0 -66px; }

.country-flag.flag-ht {
  background-position: -16px -66px; }

.country-flag.flag-hu {
  background-position: -32px -66px; }

.country-flag.flag-ic {
  background-position: -48px -66px; }

.country-flag.flag-id {
  background-position: -64px -66px; }

.country-flag.flag-ie {
  background-position: -80px -66px; }

.country-flag.flag-il {
  background-position: -96px -66px; }

.country-flag.flag-im {
  background-position: -112px -66px; }

.country-flag.flag-in {
  background-position: -128px -66px; }

.country-flag.flag-io {
  background-position: -144px -66px; }

.country-flag.flag-iq {
  background-position: -160px -66px; }

.country-flag.flag-ir {
  background-position: -176px -66px; }

.country-flag.flag-is {
  background-position: -192px -66px; }

.country-flag.flag-it {
  background-position: -208px -66px; }

.country-flag.flag-je {
  background-position: -224px -66px; }

.country-flag.flag-jm {
  background-position: -240px -66px; }

.country-flag.flag-jo {
  background-position: 0 -77px; }

.country-flag.flag-jp {
  background-position: -16px -77px; }

.country-flag.flag-ke {
  background-position: -32px -77px; }

.country-flag.flag-kg {
  background-position: -48px -77px; }

.country-flag.flag-kh {
  background-position: -64px -77px; }

.country-flag.flag-ki {
  background-position: -80px -77px; }

.country-flag.flag-km {
  background-position: -96px -77px; }

.country-flag.flag-kn {
  background-position: -112px -77px; }

.country-flag.flag-kp {
  background-position: -128px -77px; }

.country-flag.flag-kr {
  background-position: -144px -77px; }

.country-flag.flag-kurdistan {
  background-position: -160px -77px; }

.country-flag.flag-kw {
  background-position: -176px -77px; }

.country-flag.flag-ky {
  background-position: -192px -77px; }

.country-flag.flag-kz {
  background-position: -208px -77px; }

.country-flag.flag-la {
  background-position: -224px -77px; }

.country-flag.flag-lb {
  background-position: -240px -77px; }

.country-flag.flag-lc {
  background-position: 0 -88px; }

.country-flag.flag-li {
  background-position: -16px -88px; }

.country-flag.flag-lk {
  background-position: -32px -88px; }

.country-flag.flag-lr {
  background-position: -48px -88px; }

.country-flag.flag-ls {
  background-position: -64px -88px; }

.country-flag.flag-lt {
  background-position: -80px -88px; }

.country-flag.flag-lu {
  background-position: -96px -88px; }

.country-flag.flag-lv {
  background-position: -112px -88px; }

.country-flag.flag-ly {
  background-position: -128px -88px; }

.country-flag.flag-ma {
  background-position: -144px -88px; }

.country-flag.flag-mc {
  background-position: -160px -88px; }

.country-flag.flag-md {
  background-position: -176px -88px; }

.country-flag.flag-me {
  background-position: -192px -88px; }

.country-flag.flag-mg {
  background-position: -208px -88px; }

.country-flag.flag-mh {
  background-position: -224px -88px; }

.country-flag.flag-mk {
  background-position: -240px -88px; }

.country-flag.flag-ml {
  background-position: 0 -99px; }

.country-flag.flag-mm {
  background-position: -16px -99px; }

.country-flag.flag-mn {
  background-position: -32px -99px; }

.country-flag.flag-mo {
  background-position: -48px -99px; }

.country-flag.flag-mp {
  background-position: -64px -99px; }

.country-flag.flag-mq {
  background-position: -80px -99px; }

.country-flag.flag-mr {
  background-position: -96px -99px; }

.country-flag.flag-ms {
  background-position: -112px -99px; }

.country-flag.flag-mt {
  background-position: -128px -99px; }

.country-flag.flag-mu {
  background-position: -144px -99px; }

.country-flag.flag-mv {
  background-position: -160px -99px; }

.country-flag.flag-mw {
  background-position: -176px -99px; }

.country-flag.flag-mx {
  background-position: -192px -99px; }

.country-flag.flag-my {
  background-position: -208px -99px; }

.country-flag.flag-mz {
  background-position: -224px -99px; }

.country-flag.flag-na {
  background-position: -240px -99px; }

.country-flag.flag-nc {
  background-position: 0 -110px; }

.country-flag.flag-ne {
  background-position: -16px -110px; }

.country-flag.flag-nf {
  background-position: -32px -110px; }

.country-flag.flag-ng {
  background-position: -48px -110px; }

.country-flag.flag-ni {
  background-position: -64px -110px; }

.country-flag.flag-nl {
  background-position: -80px -110px; }

.country-flag.flag-no {
  background-position: -96px -110px; }

.country-flag.flag-np {
  background-position: -112px -110px; }

.country-flag.flag-nr {
  background-position: -128px -110px; }

.country-flag.flag-nu {
  background-position: -144px -110px; }

.country-flag.flag-nz {
  background-position: -160px -110px; }

.country-flag.flag-om {
  background-position: -176px -110px; }

.country-flag.flag-pa {
  background-position: -192px -110px; }

.country-flag.flag-pe {
  background-position: -208px -110px; }

.country-flag.flag-pf {
  background-position: -224px -110px; }

.country-flag.flag-pg {
  background-position: -240px -110px; }

.country-flag.flag-ph {
  background-position: 0 -121px; }

.country-flag.flag-pk {
  background-position: -16px -121px; }

.country-flag.flag-pl {
  background-position: -32px -121px; }

.country-flag.flag-pm {
  background-position: -48px -121px; }

.country-flag.flag-pn {
  background-position: -64px -121px; }

.country-flag.flag-pr {
  background-position: -80px -121px; }

.country-flag.flag-ps {
  background-position: -96px -121px; }

.country-flag.flag-pt {
  background-position: -112px -121px; }

.country-flag.flag-pw {
  background-position: -128px -121px; }

.country-flag.flag-py {
  background-position: -144px -121px; }

.country-flag.flag-qa {
  background-position: -160px -121px; }

.country-flag.flag-re {
  background-position: -176px -121px; }

.country-flag.flag-ro {
  background-position: -192px -121px; }

.country-flag.flag-rs {
  background-position: -208px -121px; }

.country-flag.flag-ru {
  background-position: -224px -121px; }

.country-flag.flag-rw {
  background-position: -240px -121px; }

.country-flag.flag-sa {
  background-position: 0 -132px; }

.country-flag.flag-sb {
  background-position: -16px -132px; }

.country-flag.flag-sc {
  background-position: -32px -132px; }

.country-flag.flag-scotland {
  background-position: -48px -132px; }

.country-flag.flag-sd {
  background-position: -64px -132px; }

.country-flag.flag-se {
  background-position: -80px -132px; }

.country-flag.flag-sg {
  background-position: -96px -132px; }

.country-flag.flag-sh {
  background-position: -112px -132px; }

.country-flag.flag-si {
  background-position: -128px -132px; }

.country-flag.flag-sk {
  background-position: -144px -132px; }

.country-flag.flag-sl {
  background-position: -160px -132px; }

.country-flag.flag-sm {
  background-position: -176px -132px; }

.country-flag.flag-sn {
  background-position: -192px -132px; }

.country-flag.flag-so {
  background-position: -208px -132px; }

.country-flag.flag-somaliland {
  background-position: -224px -132px; }

.country-flag.flag-sr {
  background-position: -240px -132px; }

.country-flag.flag-ss {
  background-position: 0 -143px; }

.country-flag.flag-st {
  background-position: -16px -143px; }

.country-flag.flag-sv {
  background-position: -32px -143px; }

.country-flag.flag-sx {
  background-position: -48px -143px; }

.country-flag.flag-sy {
  background-position: -64px -143px; }

.country-flag.flag-sz {
  background-position: -80px -143px; }

.country-flag.flag-tc {
  background-position: -96px -143px; }

.country-flag.flag-td {
  background-position: -112px -143px; }

.country-flag.flag-tf {
  background-position: -128px -143px; }

.country-flag.flag-tg {
  background-position: -144px -143px; }

.country-flag.flag-th {
  background-position: -160px -143px; }

.country-flag.flag-tj {
  background-position: -176px -143px; }

.country-flag.flag-tk {
  background-position: -192px -143px; }

.country-flag.flag-tl {
  background-position: -208px -143px; }

.country-flag.flag-tm {
  background-position: -224px -143px; }

.country-flag.flag-tn {
  background-position: -240px -143px; }

.country-flag.flag-to {
  background-position: 0 -154px; }

.country-flag.flag-tr {
  background-position: -16px -154px; }

.country-flag.flag-tt {
  background-position: -32px -154px; }

.country-flag.flag-tv {
  background-position: -48px -154px; }

.country-flag.flag-tw {
  background-position: -64px -154px; }

.country-flag.flag-tz {
  background-position: -80px -154px; }

.country-flag.flag-ua {
  background-position: -96px -154px; }

.country-flag.flag-ug {
  background-position: -112px -154px; }

.country-flag.flag-um {
  background-position: -128px -154px; }

.country-flag.flag-us {
  background-position: -144px -154px; }

.country-flag.flag-uy {
  background-position: -160px -154px; }

.country-flag.flag-uz {
  background-position: -176px -154px; }

.country-flag.flag-va {
  background-position: -192px -154px; }

.country-flag.flag-vc {
  background-position: -208px -154px; }

.country-flag.flag-ve {
  background-position: -224px -154px; }

.country-flag.flag-vg {
  background-position: -240px -154px; }

.country-flag.flag-vi {
  background-position: 0 -165px; }

.country-flag.flag-vn {
  background-position: -16px -165px; }

.country-flag.flag-vu {
  background-position: -32px -165px; }

.country-flag.flag-wales {
  background-position: -48px -165px; }

.country-flag.flag-wf {
  background-position: -64px -165px; }

.country-flag.flag-ws {
  background-position: -80px -165px; }

.country-flag.flag-ye {
  background-position: -96px -165px; }

.country-flag.flag-yt {
  background-position: -112px -165px; }

.country-flag.flag-za {
  background-position: -128px -165px; }

.country-flag.flag-zanzibar {
  background-position: -144px -165px; }

.country-flag.flag-zm {
  background-position: -160px -165px; }

.country-flag.flag-zw {
  background-position: -176px -165px; }

#scrollUp {
  background-image: url("../images/top.png");
  bottom: 20px;
  right: 20px;
  width: 38px;
  height: 38px; }

.tt-input {
  -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
  -moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
  box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075); }

.tt-hint {
  color: #999; }

.tt-menu {
  /* used to be tt-dropdown-menu in older versions */
  width: 422px;
  margin-top: 4px;
  padding: 4px 0;
  background-color: #fff;
  border: 1px solid #ccc;
  border: 1px solid rgba(0, 0, 0, 0.2);
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
  -moz-box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
  box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2); }

.tt-suggestion {
  padding: 3px 20px;
  line-height: 24px; }

.tt-suggestion.tt-cursor, .tt-suggestion:hover {
  color: #fff;
  background-color: #0097cf; }

.tt-suggestion p {
  margin: 0; }

.horizontal-scroll {
  overflow-x: auto; }

@media (min-width: 320px) {
  .pull-xs-left {
    float: left !important; }
  .pull-xs-right {
    float: right !important; }
  .pull-xs-none {
    float: none !important; } }

@media (min-width: 768px) {
  .pull-sm-left {
    float: left !important; }
  .pull-sm-right {
    float: right !important; }
  .pull-sm-none {
    float: none !important; } }

@media (min-width: 992px) {
  .pull-md-left {
    float: left !important; }
  .pull-md-right {
    float: right !important; }
  .pull-md-none {
    float: none !important; } }

@media (min-width: 1200px) {
  .pull-lg-left {
    float: left !important; }
  .pull-lg-right {
    float: right !important; }
  .pull-lg-none {
    float: none !important; } }

@media (max-width: 35em) {
  /* css for reflow & reflow2 widgets */
  .ui-table-reflow td,
  .ui-table-reflow th {
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
    float: right;
    /* if not using the stickyHeaders widget (not the css3 version)
         * the "!important" flag, and "height: auto" can be removed */
    width: 100% !important;
    height: auto !important; }
  /* reflow widget only */
  .ui-table-reflow tbody td[data-title]:before {
    color: #469;
    font-size: .9em;
    content: attr(data-title);
    float: left;
    width: 50%;
    white-space: pre-wrap;
    text-align: bottom;
    display: inline-block; }
  /* reflow2 widget only */
  table.ui-table-reflow .ui-table-cell-label.ui-table-cell-label-top {
    display: block;
    padding: .4em 0;
    margin: .4em 0;
    text-transform: uppercase;
    font-size: .9em;
    font-weight: 400; }
  table.ui-table-reflow .ui-table-cell-label {
    padding: .4em;
    min-width: 30%;
    display: inline-block;
    margin: -.4em 1em -.4em -.4em; } }
